@@ -11,6 +11,8 @@ import networkx as nx
 import autonetkit.ank as ank_utils
 import itertools
 
+#TODO: revisit phy_neighbors for eg ASN and use l3_conn instead
+
 #TODO: remove retain edge_id once removed from compiler
 #TODO: note that build network now assumes input graph has interface mappings on nodes/edges
 
@@ -108,6 +110,32 @@ def initialise(input_graph):
     autonetkit.update_http(anm)
     return anm
 
+def check_server_asns(anm):
+    """Checks that servers have appropriate ASN allocated.
+    Warns and auto-corrects servers which are connected to routers of a difference AS.
+    #TODO: provide manual over-ride for this auto-correct.
+    """
+    g_phy = anm['phy']
+
+    for server in g_phy.nodes('is_server'):
+        l3_neighbors = list(server['l3_conn'].neighbors())
+        l3_neighbor_asns = set(n.asn for n in l3_neighbors)
+        if server.asn not in l3_neighbor_asns:
+            neighs_with_asn = ["%s: AS %s" % (n, n.asn)
+                for n in l3_neighbors] # tuples for warning message
+            log.warning("Server %s does not belong to same ASN as neighbors %s" % (server, neighs_with_asn))
+
+            if len(l3_neighbors) == 1:
+                # single ASN of neighbor -> auto correct
+                if server['input'].default_asn:
+                    neigh_asn = l3_neighbor_asns.pop()
+                    log.warning("Updating server %s AS from %s to %s" % (server, server.asn, neigh_asn))
+                    server.asn = neigh_asn
+                else:
+                    log.info("Server %s ASN %s explictly set by user, not auto-correcting" %
+                        (server, server.asn))
+
+
 def apply_design_rules(anm):
     """Applies appropriate design rules to ANM"""
     g_in = anm['input']
@@ -119,6 +147,9 @@ def apply_design_rules(anm):
     autonetkit.update_http(anm)
     build_l3_connectivity(anm)
 
+    check_server_asns(anm)
+    autonetkit.update_http(anm)
+
     build_vrf(anm) # need to do before to add loopbacks before ip allocations
     from autonetkit.design.ip import build_ip, build_ipv4,build_ipv6
     build_ip(anm) # ip infrastructure topology
@@ -131,7 +162,6 @@ def apply_design_rules(anm):
         log.info("IP addressing disabled, disabling routing protocol configuration")
         anm['phy'].data.enable_routing = False
 
-
     if address_family == "None":
         log.info("IP addressing disabled, skipping IPv4")
         anm.add_overlay("ipv4") # create empty so rest of code follows through
@@ -178,6 +208,10 @@ def apply_design_rules(anm):
     autonetkit.design.bgp.build_bgp(anm)
     autonetkit.update_http(anm)
 
+    import autonetkit.design.mpls
+    autonetkit.design.mpls.mpls_te(anm)
+    autonetkit.design.mpls.mpls_oam(anm)
+
 # post-processing
     if anm['phy'].data.enable_routing:
         mark_ebgp_vrf(anm)
@@ -462,6 +496,7 @@ def build_phy(anm):
         ank_utils.copy_attr_from(g_in, g_phy, "use_onepk")
         ank_utils.copy_attr_from(g_in, g_phy, "label_full")
         ank_utils.copy_attr_from(g_in, g_phy, "indices")
+        ank_utils.copy_attr_from(g_in, g_phy, "dont_configure_static_routing")
 
     g_phy.allocate_interfaces()
 
@@ -495,6 +530,8 @@ def build_l3_connectivity(anm):
                             retain="edge_id")
     for edge in exploded_edges:
         edge.multipoint = True
+        edge.src_int.multipoint = True
+        edge.dst_int.multipoint = True
 
 def build_conn(anm):
     """Build connectivity overlay"""
