@@ -149,6 +149,12 @@ def worker():
         hostname = ""
         success = False
         result = str(e)
+        if "No route to host" in e:
+          # simpler message
+          result = "No route to host"
+        if "pexpect.TIMEOUT" in str(e):
+          #TODO: test for timeout exception directly
+          result = "Pexpect timeout"
        finally:
         try:
           data = str(data)
@@ -165,7 +171,7 @@ def worker():
           print "Sent to zmq"
 
 def main():
-  num_worker_threads = 1
+  num_worker_threads = 5
   try:
     num_worker_threads = int(sys.argv[1])
   except IndexError:
