@@ -73,16 +73,86 @@ class IosBaseCompiler(RouterCompiler):
         for interface in node.physical_interfaces:
             interface.use_cdp = node.use_cdp # use node value
 
+    def mpls_oam(self, node):
+        g_mpls_oam = self.anm['mpls_oam']
+        if node not in g_mpls_oam:
+            return # no mpls oam configured
+
+        # Node is present in mpls OAM
+        node.mpls.oam = True
+
+    def mpls_te(self, node):
+        g_mpls_te = self.anm['mpls_te']
+
+        if node not in g_mpls_te:
+            return # no mpls te configured
+
+        node.mpls_te = True
+
+        if node.isis:
+            node.isis.ipv4_mpls_te = True
+            node.isis.mpls_te_router_id = self.lo_interface
+
+        if node.ospf:
+            node.ospf.ipv4_mpls_te = True
+            node.ospf.mpls_te_router_id = self.lo_interface
+
+    def nailed_up_routes(self, node):
+        log.debug("Configuring nailed up routes")
+        phy_node = self.anm['phy'].node(node)
+        node.bgp.ipv4_nailed_up_routes = []
+        node.bgp.ipv6_nailed_up_routes = []
+
+        if node.is_ebgp_v4 and node.ip.use_ipv4:
+            infra_blocks = self.anm['ipv4'].data['infra_blocks'].get(phy_node.asn) or []
+            for infra_route in infra_blocks:
+                node.bgp.ipv4_nailed_up_routes.append(infra_route)
+
+        if node.is_ebgp_v6 and node.ip.use_ipv6:
+            infra_blocks = self.anm['ipv6'].data['infra_blocks'].get(phy_node.asn) or []
+            for infra_route in infra_blocks:
+                node.bgp.ipv6_nailed_up_routes.append(infra_route)
+
     def bgp(self, node):
         node.bgp.lo_interface = self.lo_interface
         super(IosBaseCompiler, self).bgp(node)
+        phy_node = self.anm['phy'].node(node)
+        asn = phy_node.asn
+        g_ebgp_v4 = self.anm['ebgp_v4']
+        g_ebgp_v6 = self.anm['ebgp_v6']
 
-        # Only advertise loopbacks into eBGP
+        if node in g_ebgp_v4 and len(list(g_ebgp_v4.node(node).edges())) > 0:
+            node.is_ebgp_v4 = True
+        else:
+            node.is_ebgp_v4 = False
+
+        if node in g_ebgp_v6 and len(list(g_ebgp_v6.node(node).edges())) > 0:
+            node.is_ebgp_v6 = True
+        else:
+            node.is_ebgp_v6 = False
+
+        node.bgp.ipv4_advertise_subnets = []
+        node.bgp.ipv6_advertise_subnets = []
+
+         # Advertise loopbacks into BGP
         if node.ip.use_ipv4:
             node.bgp.ipv4_advertise_subnets = [node.loopback_zero.ipv4_cidr]
         if node.ip.use_ipv6:
             node.bgp.ipv6_advertise_subnets = [node.loopback_zero.ipv6_address]
 
+        # Advertise infrastructure into eBGP
+        if node.ip.use_ipv4 and node.is_ebgp_v4:
+            infra_blocks = self.anm['ipv4'].data['infra_blocks'].get(asn) or []
+            for infra_route in infra_blocks:
+                node.bgp.ipv4_advertise_subnets.append(infra_route)
+
+        if node.ip.use_ipv6 and node.is_ebgp_v6:
+            infra_blocks = self.anm['ipv6'].data['infra_blocks'].get(asn) or []
+            for infra_route in infra_blocks:
+                            node.bgp.ipv6_advertise_subnets.append(infra_route)
+
+        self.nailed_up_routes(node)
+
         # vrf
         #TODO: this should be inside vrf section?
         node.bgp.vrfs = []
@@ -108,13 +178,11 @@ class IosBaseCompiler(RouterCompiler):
             # eBGP sessions for this VRF
             vrf_ebgp_neighbors = defaultdict(list)
 
-            g_ebgp_v4 = self.anm['ebgp_v4']
             for session in sort_sessions(g_ebgp_v4.edges(vrf_node)):
                 if session.exclude and session.vrf:
                     data = self.ebgp_session_data(session, ip_version = 4)
                     vrf_ebgp_neighbors[session.vrf].append(data)
 
-            g_ebgp_v6 = self.anm['ebgp_v6']
             for session in sort_sessions(g_ebgp_v6.edges(vrf_node)):
                 if session.exclude and session.vrf:
                     data = self.ebgp_session_data(session, ip_version = 6)
@@ -140,6 +208,8 @@ class IosBaseCompiler(RouterCompiler):
                 retain = True
             node.bgp.vpnv4 = {'retain_route_target': retain}
 
+
+
     def vrf_igp_interfaces(self, node):
         # marks physical interfaces to exclude from IGP
         vrf_node = self.anm['vrf'].node(node)
@@ -285,11 +355,16 @@ class IosClassicCompiler(IosBaseCompiler):
 
     def compile(self, node):
         super(IosClassicCompiler, self).compile(node)
+
+        self.mpls_te(node)
+        self.mpls_oam(node)
+
         phy_node = self.anm['phy'].node(node)
         if phy_node.device_subtype == "vios":
             # only copy across for certain reference platforms
             node.use_onepk = phy_node.use_onepk
             node.no_service_config = True
+            node.ipv6_cef = True
 
         if phy_node.device_subtype == "CSR1000v":
             # only copy across for certain reference platforms
@@ -297,6 +372,7 @@ class IosClassicCompiler(IosBaseCompiler):
             node.include_csr = True
             # Set secret password to "cisco"
             node.enable_secret = "tnhtc92DXBhelxjYk8LWJrPV36S2i4ntXrpb4RFmfqY"
+            node.exclude_phy_int_auto_speed_duplex = True
 
     def ospf(self, node):
         super(IosClassicCompiler, self).ospf(node)
@@ -311,9 +387,87 @@ class IosClassicCompiler(IosBaseCompiler):
                         'multipoint': False,
                         } #TODO: add wrapper for this
 
+    def mpls_te(self, node):
+        super(IosClassicCompiler, self).mpls_te(node)
+
+        g_mpls_te = self.anm['mpls_te']
+        if node not in g_mpls_te:
+            return # no mpls te configured
+
+        mpls_te_node = g_mpls_te.node(node)
+
+        for interface in mpls_te_node.physical_interfaces:
+            nidb_interface = self.nidb.interface(interface)
+            if not interface.is_bound:
+                log.debug("Not enable MPLS and RSVP for "
+                    "interface %s on %s "% (nidb_interface.id, node))
+                continue
+            nidb_interface.te_tunnels = True
+            nidb_interface.rsvp_bandwidth_percent = 100
+
     def bgp(self, node):
         super(IosClassicCompiler, self).bgp(node)
 
+        node.bgp.use_ipv4 = node.ip.use_ipv4
+        node.bgp.use_ipv6 = node.ip.use_ipv6
+
+        # Seperate by address family
+        ipv4_peers = []
+        ipv6_peers = []
+        #Note cast to dict - #TODO revisit this requirement
+        #TODO: revisit and tidy up the logic here: split iBGP and eBGP
+        #TODO: sort the peer list by peer IP
+        for peer in node.bgp.ibgp_neighbors:
+            peer = dict(peer) # for storage back into NIDB
+            peer['remote_ip'] = peer['loopback']
+            if peer['use_ipv4']:
+                if node.is_ebgp_v4:
+                    peer['next_hop_self'] = True
+                ipv4_peers.append(peer)
+            if peer['use_ipv6']:
+                if node.is_ebgp_v6:
+                    peer['next_hop_self'] = True
+                ipv6_peers.append(peer)
+
+        for peer in node.bgp.ibgp_rr_parents:
+            peer = dict(peer) # for storage back into NIDB
+            peer['remote_ip'] = peer['loopback']
+            if peer['use_ipv4']:
+                if node.is_ebgp_v4:
+                    peer['next_hop_self'] = True
+                ipv4_peers.append(peer)
+            if peer['use_ipv6']:
+                if node.is_ebgp_v6:
+                    peer['next_hop_self'] = True
+                ipv6_peers.append(peer)
+
+        for peer in node.bgp.ibgp_rr_clients:
+            peer = dict(peer) # for storage back into NIDB
+            peer['rr_client'] = True
+            peer['remote_ip'] = peer['loopback']
+            if peer['use_ipv4']:
+                if node.is_ebgp_v4:
+                    peer['next_hop_self'] = True
+                ipv4_peers.append(peer)
+            if peer['use_ipv6']:
+                if node.is_ebgp_v6:
+                    peer['next_hop_self'] = True
+                ipv6_peers.append(peer)
+
+        for peer in node.bgp.ebgp_neighbors:
+            peer = dict(peer) # for storage back into NIDB
+            peer['is_ebgp'] = True
+            peer['remote_ip'] = peer['dst_int_ip']
+            if peer['use_ipv4']:
+                peer['next_hop_self'] = True
+                ipv4_peers.append(peer)
+            if peer['use_ipv6']:
+                peer['next_hop_self'] = True
+                ipv6_peers.append(peer)
+
+        node.bgp.ipv4_peers = ipv4_peers
+        node.bgp.ipv6_peers = ipv6_peers
+
         vpnv4_neighbors = []
         if node.bgp.vpnv4:
             for neigh in node.bgp.ibgp_neighbors:
@@ -342,8 +496,37 @@ class IosClassicCompiler(IosBaseCompiler):
         node.bgp.vpnv4_neighbors = vpnv4_neighbors
 
 class IosXrCompiler(IosBaseCompiler):
+    def compile(self, node):
+        super(IosXrCompiler, self).compile(node)
+        self.mpls_te(node)
+        self.mpls_oam(node)
+
+
+    def mpls_te(self, node):
+        super(IosXrCompiler, self).mpls_te(node)
+
+        g_mpls_te = self.anm['mpls_te']
+        if node not in g_mpls_te:
+            return # no mpls te configured
+
+        rsvp_interfaces = []
+        mpls_te_interfaces = []
+        mpls_te_node = g_mpls_te.node(node)
+
+        for interface in mpls_te_node.physical_interfaces:
+            nidb_interface = self.nidb.interface(interface)
+            rsvp_interfaces.append({
+                'id': nidb_interface.id,
+                "bandwidth_percent": 100})
+
+            mpls_te_interfaces.append(nidb_interface.id)
+
+        node.rsvp.interfaces = rsvp_interfaces
+        node.mpls.te_interfaces   = mpls_te_interfaces
+
     def ospf(self, node):
         super(IosXrCompiler, self).ospf(node)
+
         g_ospf = self.anm['ospf']
         interfaces_by_area = defaultdict(list)
 
