@@ -127,6 +127,8 @@ class CiscoCompiler(PlatformCompiler):
                 if phy_specified_id is not None:
                     interface.id = phy_specified_id
 
+        from autonetkit.compilers.device.ubuntu import UbuntuCompiler
+        ubuntu_compiler = UbuntuCompiler(self.nidb, self.anm)
         for phy_node in g_phy.nodes('is_server', host=self.host):
             #TODO: look at server syntax also, same as for routers
             nidb_node = self.nidb.node(phy_node)
@@ -136,6 +138,20 @@ class CiscoCompiler(PlatformCompiler):
                 nidb_node.ip.use_ipv6 = phy_node.use_ipv6
                 phy_int = phy_node.interface(interface)
 
+                #TODO: clean up interface handling
+                numeric_int_ids = self.numeric_interface_ids()
+                for interface in nidb_node.physical_interfaces:
+                    phy_numeric_id = phy_node.interface(interface).numeric_id
+                    if phy_numeric_id is None:
+                        #TODO: remove numeric ID code
+                        interface.numeric_id = numeric_int_ids.next()
+                    else:
+                        interface.numeric_id = int(phy_numeric_id)
+
+                    phy_specified_id = phy_node.interface(interface).specified_id
+                    if phy_specified_id is not None:
+                        interface.id = phy_specified_id
+
                 #TODO: make this part of the base device compiler, which server/router inherits
                 if nidb_node.ip.use_ipv4:
                     ipv4_int = phy_int['ipv4']
@@ -156,6 +172,19 @@ class CiscoCompiler(PlatformCompiler):
                         interface.ipv6_address = sn_preflen_to_network(ipv6_int.ip_address,
                                 interface.ipv6_subnet.prefixlen)
 
+                    # render route config
+            nidb_node = self.nidb.node(phy_node)
+            ubuntu_compiler.compile(nidb_node)
+
+            if not phy_node.dont_configure_static_routing:
+                nidb_node.render.template = os.path.join("templates", "linux", "static_route.mako")
+                if to_memory:
+                    nidb_node.render.to_memory = True
+                else:
+                    nidb_node.render.dst_folder = dst_folder
+                    nidb_node.render.dst_file = "%s.conf" % naming.network_hostname(
+                        phy_node)
+
 
         ios_compiler = IosClassicCompiler(self.nidb, self.anm)
         ios_nodes = (n for n in g_phy.nodes('is_router', host=self.host)
